#include "structure.h"
#include "outils.h"

#include <iostream> // I/O de base
#include <fstream> // I/O des fichiers
#include <sstream> // I/O de chaînes de caractères

#include <cstdlib> // Pour rand(), srand() et time()

// Macro de trace affichant un message, ainsi que la ligne et le fichier de l'appel
#define TRACE(msg) { std::ostringstream oss; { oss << msg; }; trace(__FILE__,__LINE__,oss.str()); }
inline void trace(const std::string & file, int line, const std::string & msg) {
  std::ostringstream oss;
  oss << "[DEBUG] " << msg << " [" << file << ":" << line << "]";
  std::cerr << oss.str() << std::endl;
}

using namespace std;
using namespace fraction;

//#define INTERACTIF // Pour les parties interactives au clavier
//#define ECRIRE_FRACTION // Pour générer le fichier contenant une fraction

// Déclaration des fonctions de test
void test_io(const std::string & input_path, const std::string & output_path);
void test_fraction(const std::string & input_path, const std::string & output_path);

// Programme principal
int main(int argc, char** argv)
{
  // Affichage simple
  cout << "Bonjour!" << endl;

  // Affichage des arguments du programme
  for (int i=0; i<argc; i++) {
    cout << "Arg #" << i << ": " << argv[i] << endl;
  }

  // Récupération des arguments du programme
  string inputPath = "numbers.txt";
  if (argc > 1) {
    inputPath = argv[1];
  }
  string outputPath = "sorted_numbers.txt";
  if (argc > 2) {
    outputPath = argv[2];
  }

  // Test d'I/O
  TRACE( "Lancement du test d'I/O avec " << inputPath << " et " << outputPath );
  test_io(inputPath,outputPath);

  // Test du module Fraction
  TRACE( "Lancement du test du module Fraction" );
  test_fraction("data.txt","data.txt");

  TRACE( "Fin des tests" );

  return 0;
}

// Fonction permettant d'échanger la valeur de deux variables
inline void swap(int & a, int & b) {
  int tmp = a;
  a = b;
  b = tmp;
}

// Test d'I/O
void test_io(const std::string & input_path, const std::string & output_path) {

#ifdef INTERACTIF
  int a = 0;
  int b = 0;

  // Lecture de deux entiers
  do {
    if (!cin.good()) {
      cin.clear(); // D'abord le clear !..
      cin.ignore(100,'\n'); // Ignorer cette ligne
      cerr << "Erreur..." << endl;
    }

    cout << "Entrez deux entiers: ";
    cin >> a >> b;

  } while (!cin.good()); // Jusqu'à ce que ce soit bon...

  cin.ignore(100,'\n'); // Ignorer le reste de la ligne

  // Affichage (et calcul)
  cout << "a = " << a << endl;
  cout << "b = " << b << endl;
  cout << "a + b = " << (a + b) << endl;
  cout << "a - b = " << (a - b) << endl;

  // Lecture d'une ligne
  string s;
  cout << "Entrez votre nom: ";
  getline(cin,s);
  cout << "Bonjour, " << s  << " !" << endl;

  // Recherche des initiales
  string::size_type pos = s.find(" ");
  if (pos != string::npos) {
    string initiales = s.substr(0,1) + "." + s[pos+1] + ".";
    cout << "Bonjour, " << initiales  << " !" << endl;
  }
#endif

  // Définition de constante et alias de type
  const int TAILLE = 10;
  typedef int Tableau[TAILLE];

  // Variable pour stocker les entiers
  Tableau tableau;

  // Lecture du fichier
  ifstream entree(input_path.c_str());
  for (int i=0; i<TAILLE; i++) {
    entree >> tableau[i];
  }
  entree.close();

  // Tri bulle (non-optimisé)
  for (int n=0; n<TAILLE; n++) {
    for (int i=0; i<TAILLE-1; i++) {
      if (tableau[i] > tableau[i+1]) {
        swap(tableau[i],tableau[i+1]);
      }
    }
  }

  // Ecriture dans le fichier
  ofstream sortie(output_path.c_str());
  for (int i=0; i<TAILLE; i++) {
    sortie << tableau[i] << endl;
  }
  sortie.close();

}

// Test du module Fraction
void test_fraction(const std::string & input_path, const std::string & output_path) {

//  srand ( time(NULL) );
//  for (int i=0; i<1000; i++ ) {
//    cout << (rand() % 256)+1 << " ";
//  }
//  cout << endl;

  Fraction f = creer(3,7);
  afficher(f);
  Fraction g = creer(12.57f);
  afficher(g);

  Fraction h = additionner( f, g );
  afficher( h );
  Fraction i = simplifier( h );
  afficher( i );

  cout << pgcd( 12, 4 ) << endl;
  cout << pgcd( 17, 23 ) << endl;
  cout << pgcd( 33, 22 ) << endl;

  Fraction a = creer(1,3);

#ifdef INTERACTIF
  cout << "Entrez une fraction: ";
  lire(cin,a);
  afficher(a);
  ecrire(cout,a);
#endif

#ifdef ECRIRE_FRACTION
  ofstream fichier_sortie(output_path.c_str());
  ecrire(fichier_sortie,a);
  fichier_sortie.close();
#endif

  Fraction b;
  ifstream fichier_entree(input_path.c_str());
  while (!fichier_entree.eof() ) {
    if (lire(fichier_entree,b)) {
      afficher(b);
      afficher(b,PARTIE_ENTIERE);
    }
  }
  fichier_entree.close();
}
